<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Storage;
use Laravelista\Algo\Models\Category;
use Illuminate\Validation\Rule;
use Laravelista\Picasso\Picasso;
use Laravelista\Algo\Helpers\Slug;

class CategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $categories = Category::with(['translations'])->get()->sortBy('name');

        return view('algo::categories.index')->withCategories($categories);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('algo::categories.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  Slug $slug
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, Slug $slug, Picasso $picasso)
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('category_translations')
                    ->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $category = new Category;
        $category->translateOrNew($locale)->name = $validatedData['name'];
        $category->translateOrNew($locale)->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $category->translateOrNew($locale)->description = $validatedData['description'];
        if ($request->hasFile('image')) {
            $category->image = $request->image->store('images', 'originals');
        }
        $category->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($category->image, config('algo.category_dimensions'), 'public');
        }

        return redirect()
            ->route('algo.categories.index')
            ->with('status', 'Kategorija napravljena!');
    }

    /**
     * Display the specified resource.
     *
     * @param  Category $category
     * @return \Illuminate\Http\Response
     */
    public function show(Category $category)
    {
        return view('algo::categories.show')->withCategory($category);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  Category $category
     * @return \Illuminate\Http\Response
     */
    public function edit(Category $category)
    {
        return view('algo::categories.edit')->withCategory($category);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  Category $category
     * @param  Slug $slug
     * @param  Storage $storage
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Category $category, Slug $slug, Storage $storage, Picasso $picasso)
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('category_translations')
                    ->ignore($category->id, 'category_id')
                    ->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $category->translateOrNew($locale)->name = $validatedData['name'];
        $category->translateOrNew($locale)->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $category->translateOrNew($locale)->description = $validatedData['description'];

        if ($request->hasFile('image') and $category->hasImage()) {
            $storage::disk('originals')->delete($category->image);
            $picasso->purge($category->image, 'public');
        }

        if ($request->hasFile('image')) {
            $category->image = $request->image->store('images', 'originals');
        }

        $category->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($category->image, config('algo.category_dimensions'), 'public');
        }

        return redirect()
            ->route('algo.categories.index')
            ->with('status', 'Kategorija uređena!');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  Category $category
     * @return \Illuminate\Http\Response
     */
    public function destroy(Category $category, Storage $storage, Picasso $picasso)
    {
        if ($category->hasImage()) {
            $storage::disk('originals')->delete($category->image);
            $picasso->purge($category->image, 'public');
        }

        $category->delete();

        return redirect()
            ->route('algo.categories.index')
            ->with('status', 'Kategorija obrisana!');
    }
}
