<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Laravelista\Algo\Models\Post;
use Laravelista\Algo\Models\Tag;
use Laravelista\Algo\Models\Category;
use Illuminate\Validation\Rule;
use Laravelista\Algo\Helpers\PublishedAt;
use Laravelista\Algo\Helpers\Slug;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Storage;
use Laravelista\Picasso\Picasso;

class PostController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $posts = Post::
            with(['translations', 'tags.translations', 'category.translations', 'author'])
            ->orderByRaw('-published_at ASC')
            ->orderBy('created_at', 'DESC')
            ->get();

        return view('algo::posts.index')->withPosts($posts);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('algo::posts.create')->with([
            'authors' => Config::get('algo.user')::pluck('name', 'id')->toArray(),
            'tags' => Tag::with(['translations'])->get()->pluck('name', 'id')->toArray(),
            'categories' => Category::with(['translations'])->get()->pluck('name', 'id')->toArray()
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param PublishedAt $publishedAt
     * @param Slug $slug
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, PublishedAt $publishedAt, Slug $slug)
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'title' => ['required', 'string', 'max:255'],
            'intro' => ['nullable', 'string', 'max:255'],
            'content' => ['required', 'string'],
            'video_link' => ['nullable', 'url', 'max:255'],
            'published_at_date' => ['nullable', 'date'],
            'published_at_time' => ['nullable', 'date_format:H:i'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('post_translations')
                    ->where('locale', $locale)
            ],
            'author_id' => ['required', 'integer', 'min:1', 'exists:users,id'],
            'category_id' => ['required', 'integer', 'min:1', 'exists:categories,id'],
            'tags_list' => ['nullable', 'array'],
            'tags_list.*' => ['required', 'integer', 'min:1', 'exists:tags,id'],
            'featured' => ['nullable', 'boolean']
        ]);

        $post = new Post;
        $post->featured = $validatedData['featured'] ?? false;
        $post->video_link = $validatedData['video_link'];
        $post->translateOrNew($locale)->title = $validatedData['title'];
        $post->translateOrNew($locale)->intro = $validatedData['intro'];
        $post->translateOrNew($locale)->content = $validatedData['content'];
        $post->translateOrNew($locale)->slug = $slug->get($validatedData['slug'], $validatedData['title']);
        $post->author()->associate($validatedData['author_id']);
        $post->category()->associate($validatedData['category_id']);
        $post->published_at = $publishedAt->get($validatedData['published_at_date'], $validatedData['published_at_time']);
        $post->save();
        $post->tags()->sync($validatedData['tags_list'] ?? []);

        return redirect()
            ->route('algo.posts.index')
            ->with('status', 'Objava napravljena!');
    }

    /**
     * Display the specified resource.
     *
     * @param  \Laravelista\Algo\Post $post
     * @return \Illuminate\Http\Response
     */
    public function show(Post $post)
    {
        $post->load(['tags.translations']);
        return view('algo::posts.show')->withPost($post);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \Laravelista\Algo\Post $post
     * @return \Illuminate\Http\Response
     */
    public function edit(Post $post)
    {
        return view('algo::posts.edit')->with([
            'post' => $post->load(['translations', 'tags', 'category', 'author']),
            'authors' => Config::get('algo.user')::pluck('name', 'id')->toArray(),
            'tags' => Tag::with(['translations'])->get()->pluck('name', 'id')->toArray(),
            'categories' => Category::with(['translations'])->get()->pluck('name', 'id')->toArray()
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Laravelista\Algo\Post $post
     * @param  PublishedAt $publishedAt
     * @param  Slug $slug
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Post $post, PublishedAt $publishedAt, Slug $slug)
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'title' => ['required', 'string', 'max:255'],
            'intro' => ['nullable', 'string', 'max:255'],
            'content' => ['required', 'string'],
            'video_link' => ['nullable', 'url', 'max:255'],
            'published_at_date' => ['nullable', 'date'],
            'published_at_time' => ['nullable', 'date_format:H:i'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('post_translations')
                    ->ignore($post->id, 'post_id')
                    ->where('locale', $locale)
            ],
            'author_id' => ['required', 'integer', 'min:1', 'exists:users,id'],
            'category_id' => ['required', 'integer', 'min:1', 'exists:categories,id'],
            'tags_list' => ['nullable', 'array'],
            'tags_list.*' => ['required', 'integer', 'min:1', 'exists:tags,id'],
            'featured' => ['nullable', 'boolean']
        ]);

        $post->featured = $validatedData['featured'] ?? false;
        $post->video_link = $validatedData['video_link'];
        $post->translateOrNew($locale)->title = $validatedData['title'];
        $post->translateOrNew($locale)->intro = $validatedData['intro'];
        $post->translateOrNew($locale)->content = $validatedData['content'];
        $post->translateOrNew($locale)->slug = $slug->get($validatedData['slug'], $validatedData['title']);
        $post->author()->associate($validatedData['author_id']);
        $post->category()->associate($validatedData['category_id']);
        $post->published_at = $publishedAt->get($validatedData['published_at_date'], $validatedData['published_at_time']);
        $post->save();
        $post->tags()->sync($validatedData['tags_list'] ?? []);

        return redirect()
            ->route('algo.posts.index')
            ->with('status', 'Objava uređena!');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \Laravelista\Algo\Post $post
     * @return \Illuminate\Http\Response
     */
    public function destroy(Post $post, Storage $storage, Picasso $picasso)
    {
        // delete all post images before deleting the post.
        foreach ($post->images as $image) {
            $storage::disk('originals')->delete($image->path);
            $picasso->purge($image->path, 'public');
            $image->delete();
        }

        $post->delete();

        return redirect()
            ->route('algo.posts.index')
            ->with('status', 'Objava obrisana!');
    }
}
